#include <qiknumbereditor.h>	// CQikNumberEditor
#include <eikcapc.h>			// CEikCaptionedControl
#include <eikchkbx.h>			// CEikCheckBox
#include <QikSelectMediaFileDlg.h>	// CQikSelectMediaFileDlg
#include <qikutils.h>			// QikFileUtils
#include <qikzoomdlg.h>		// CQikZoomDialog
#include <eikseced.h>			// CEikSecretEditor

#include "bjackview.h"
#include "sman.hrh"

#if !defined(__DIALOGS_H__)
#define __DIALOGS_H__

// Note: this is always ONE more than the controls defined in the resource file because
// item #4 will be for SMan's autostart
#define MAX_AUTOSTART 4

/*************************************************************
*
* Logging level dialog
*
**************************************************************/

class CLogLevelDialog : public CEikDialog
{
public:
	CLogLevelDialog(TUint *logFlags);
	~CLogLevelDialog();
protected:
//	TBool ShutL() { return EFalse; };
	TUint *localLogFlags;
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
};

/*************************************************************
*
* Various bluejack timer options dialog
*
**************************************************************/

class CBluejackTimerDialog : public CEikDialog
{
public:
	CBluejackTimerDialog(TInt *noDevicePause, TInt *noDeviceAttempts, TInt *decayTime, TInt *obexTimeout);
	~CBluejackTimerDialog();
protected:
//	TBool ShutL() { return EFalse; };
	TInt *localNoDevicePause;
	TInt *localNoDeviceAttempts;
	TInt *localDecayTime;
	TInt *localObexTimeout;
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
};

/*************************************************************
*
* Hotkey dialog
*
**************************************************************/

class CHotkeyDialog : public CEikDialog
{
public:
	CHotkeyDialog(TInt *iHotkey);
	~CHotkeyDialog();
protected:
//	TBool ShutL() { return EFalse; };
	TInt *localiHotkey;
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
};

/*************************************************************
*
* Shortcuts dialog
*
**************************************************************/

class CShortcutDialog : public CEikDialog
{
public:
	CShortcutDialog(CArrayFixFlat<TUid> *shortCut);
	~CShortcutDialog();
	CDesCArray *appListArray;
	CArrayFixFlat<TUid> *appListUid;
	void populateAppArray(void);
protected:
//	TBool ShutL() { return EFalse; };
	CArrayFixFlat<TUid> *tempUid;
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
};

/*************************************************************
*
* Autostart dialog
*
**************************************************************/

class CAutoStartConfig
{
public:
	TFileName autoStartApp[MAX_AUTOSTART];
	TUid autoStartUid[MAX_AUTOSTART];
	TFileName autoStartDefaultDoc[MAX_AUTOSTART];
	TInt placeHolder;
};

class CAutostartDialog : public CEikDialog
{
public:
	CAutostartDialog();
	~CAutostartDialog();
	CAutoStartConfig autoStartConfig;
	TUint loadBootFile(void);
	TUint saveBootFile(void);
protected:
//	TBool ShutL() { return EFalse; };
	CDesCArray *appListCaption;
	CArrayFixSeg<TFileName> *appListPath;
	CArrayFixFlat<TUid> *appListUid;
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
	void populateAppArray(void);
};

/*************************************************************
*
* Bluejack notifications dialog
*
**************************************************************/

class CBluejackNotifications : public CEikDialog
{
public:
	CBluejackNotifications(CConfig *cData);
	~CBluejackNotifications();
protected:
//	TBool ShutL() { return EFalse; };
	TFileName selectedAudioFileName;
	void HandleControlStateChangeL(TInt aControlId);
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aButtonId);
	void pickAudioFile(void);
private:
	CConfig *configData;
};

/*************************************************************
*
* Bluebeam log dialog
*
**************************************************************/

class CBluebeamLogDialog : public CEikDialog
{
public:
	CBluebeamLogDialog(CLogger *logObj);
	~CBluebeamLogDialog();
protected:
	CLogger *localLogObj;
	void PostLayoutDynInitL();	
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aButtonId);
};

/*************************************************************
*
* Class for dialog for editing bluejack vcard
*
**************************************************************/

class CBluejackMessageDialog : public CEikDialog
{
public:
	CBluejackMessageDialog(CScanner *scannerObj);
	~CBluejackMessageDialog();
protected:
//	TBool ShutL() { return EFalse; };
	void makeVCard(void);
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
	CScanner *localScannerObj;
	TBuf<MAX_VCARD_SIZE> currBJackMsg;
};

/*************************************************************
*
* Fileman log dialog
*
**************************************************************/

class CFileManLogDialog : public CEikDialog
{
public:
	TInt progressValue;
	TPtrC text;
	TBool showOverwrite;
protected:
	TBool ShutL() { return EFalse; };
	void PostLayoutDynInitL();
};

/*************************************************************
*
* Fileman rename
*
**************************************************************/

class CFileManRenDialog : public CEikDialog
{
public:
	TFileName fileName;
	TPath filePath;
private:
	TFileName newName;
protected:
//	TBool ShutL() { return EFalse; };
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
};

/*************************************************************
*
* Fileman new folder
*
**************************************************************/

class CFileManNewDirDialog : public CEikDialog
{
public:
	TPath currentPath;
protected:
//	TBool ShutL() { return EFalse; };
	TBool OkToExitL(TInt aButtonId);
};

/*************************************************************
*
* Fileman attribute dialog
*
**************************************************************/

class CFileManAttrDialog : public CEikDialog
{
public:
	CFileManAttrDialog();
	~CFileManAttrDialog();
	TFileName theFile;
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

protected:
//	TBool ShutL() { return EFalse; };
	TBool OkToExitL(TInt aButtonId);
	void PostLayoutDynInitL();	
private:
	RFs fs;	
};

/*************************************************************
*
* Fn menu toggling
*
**************************************************************/

class CFnMenuToggle : public CEikDialog
{
public:
	CFnMenuToggle(TUint *bits, TInt *defaultView);
	~CFnMenuToggle();
protected:
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();	
private:
	TUint *toggleBits;
	TInt *theDefaultView;
};

/*************************************************************
*
* CRC progress
*
**************************************************************/

class CCRCDialog : public CEikDialog
{
public:
	CCRCDialog(TPtrC fileName);
	~CCRCDialog();
	void updateProgress(TInt aProgress);
	TPtrC theFileName;

protected:
	void stopCRC();
	void PreLayoutDynInitL();
};

/*************************************************************
*
* DB progress
*
**************************************************************/

class CDBProgressDialog : public CEikDialog
{
public:
	CDBProgressDialog();
	~CDBProgressDialog();
	void updateProgress(TInt aProgress);
	void setProgressMaxValue(TInt aMaxValue);

protected:
	TBool ShutL() { return EFalse; };		// Cannot be dispatched
	void PostLayoutDynInitL();
};

/*************************************************************
*
* Generic dialog to get password
*
**************************************************************/

class CGetPasswordDialog : public CEikDialog
{
public:
	CGetPasswordDialog(TPassword *thePassword);
	~CGetPasswordDialog();

protected:
	TBool OkToExitL(TInt aButtonId);

private:
	TPassword *localPassword;
};

/*************************************************************
*
* Generic dialog to set password
*
**************************************************************/

class CSetPasswordDialog : public CEikDialog
{
public:
	CSetPasswordDialog(TPassword *thePassword);
	~CSetPasswordDialog();

protected:
	TBool OkToExitL(TInt aButtonId);

private:
	TPassword *localPassword;
};

/*************************************************************
*
* Detailed calendar dialog
*
**************************************************************/

class CAgendaDetail : public CEikDialog
{
public:
	CAgendaDetail();
	~CAgendaDetail();

	TDesC *startDateTime, *endDateTime, *alarmDateTime, *location;
	TPtrC notes;
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);
	
protected:
	void PreLayoutDynInitL();
	void PostLayoutDynInitL();
};

/*************************************************************
*
* Detailed todo dialog
*
**************************************************************/

class CTodoDetail : public CEikDialog
{
public:
	CTodoDetail();
	~CTodoDetail();

	TDesC *dueDateTime, *alarmDateTime, *priority;
	TPtrC notes;
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

protected:
	void PreLayoutDynInitL();
	void PostLayoutDynInitL();
};

#endif