#include "SMan.h"
#include <SMan.rsg>
#include <cntdb.h>

//#include <etelmm.h>
//#include <etel.h>

/*************************************************************
*
* Application object
*
**************************************************************/

CApaDocument* CSMan2Application::CreateDocumentL()
{
	CSMan2Document* doc = new (ELeave) CSMan2Document(*this);
	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();
	return doc;
}

/*************************************************************
*
* Document object
*
**************************************************************/

CSMan2Document* CSMan2Document::NewL(CEikApplication& aApp)
{
	CSMan2Document* self = new (ELeave) CSMan2Document(aApp);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

CSMan2Document::CSMan2Document(CEikApplication& aApp)
		: CQikDocument(aApp)
{
}

CSMan2Document::~CSMan2Document()
{
}

TInt CSMan2Document::loadConfig(void)
{
	RFile inStream;
	TInt retVal;
	
	retVal = KErrGeneral;
	if (inStream.Open(CEikonEnv::Static()->FsSession(), configFileName, EFileRead) == KErrNone)
	{
		TInt fileSize;
		
		inStream.Size(fileSize);
		if (fileSize >= (TInt)sizeof(verData))
		{
			CVerInfo verData;
			
			// Check config version. If correct, load it else ignore it
			
			TPckg<CVerInfo> pkgVerData(verData);
			if (inStream.Read(pkgVerData) == KErrNone)
			{
				if ((verData.majorVersion == VER_MAJOR) && (verData.minorVersion == VER_MINOR))
				{
					TPckg<CConfig> pkgConfigData(configData);
					if (inStream.Read(pkgConfigData) == KErrNone)
						retVal = KErrNone;
				}
			}
		}
		inStream.Close();
	}
	return retVal;
}

void CSMan2Document::saveConfig(void)
{
	RFile outStream;
	TInt retVal;
	
	retVal = KErrGeneral;
	if (outStream.Replace(CEikonEnv::Static()->FsSession(), configFileName, EFileRead) == KErrNone)
	{
		verData.majorVersion = VER_MAJOR;
		verData.minorVersion = VER_MINOR;
		TPckg<CVerInfo> pkgVerData(verData);
		if (outStream.Write(pkgVerData) == KErrNone)
		{
			TPckg<CConfig> pkgConfigData(configData);
			if (outStream.Write(pkgConfigData) == KErrNone)
				retVal = KErrNone;
		}
		outStream.Close();
	}
	if (retVal != KErrNone)
		CEikonEnv::Static()->InfoMsg(R_TBUF_ERRSAVECONFIG);
}

void CSMan2Document::ConstructL()
{
	for (int i = 0; i < NUM_SHORTCUTS; i++)
		configData.shortCutUid[i] = KNullUid;
	configData.autoSave = CONFIG_AUTOSAVE;
	configData.hotkeyEnabled = CONFIG_HOTKEY_ENABLED;
	configData.resolveName = CONFIG_RESOLVE_HOSTNAME;
	configData.showBluejackSysMsg = CONFIG_SHOW_BLUEJACK_SYSMSG;
	configData.proxSweepOnce = CONFIG_PROX_SWEEP_ONCE;
	configData.flipFlush = CONFIG_FLIPFLUSH;
	configData.forceEndTask = CONFIG_FORCE_ENDTASK;
	configData.noDevicePause = CONFIG_NO_DEVICE_PAUSE;
	configData.noDeviceAttempts = CONFIG_NO_DEVICE_ATTEMPTS;
	configData.showHiddenTasks = CONFIG_SHOW_HIDDEN_TASKS;
	configData.decayTime = CONFIG_DECAY_TIME;
	configData.logFlags = CLogger::logAll;
	configData.obexTimeout = CONFIG_OBEX_TIMEOUT;
	configData.vibraNotify = CONFIG_VIBRA_NOTIFY;
	configData.audioFileName.Copy(_L(""));
	configData.persist = CONFIG_PERSIST;
	configData.beamProtocol = CONFIG_BEAM_PROTOCOL;
	configData.showHiddenFiles = CONFIG_FILEMAN_SHOWHIDDEN;
	configData.zoomMain = CONFIG_ZOOM_MAIN;
	configData.zoomFile = CONFIG_ZOOM_FILE;
	configData.zoomBJack = CONFIG_ZOOM_BJACK;
	configData.zoomDB = CONFIG_ZOOM_DB;
	configData.zoomSysInfo = CONFIG_ZOOM_SYSINFO;
	configData.autoRunApp = CONFIG_AUTO_RUNAPP;
	configData.autoOpenFile = CONFIG_AUTO_OPENFILE;
	configData.fnActivatedMenus = CONFIG_FN_ACTIVATED_MENUS;
	configData.dbSortAscending = CONFIG_DB_SORT_ASCENDING;
	configData.zoomAgenda = CONFIG_ZOOM_AGENDA;
	configData.agendaShowCompleteCalendar = CONFIG_AGENDA_SHOW_COMPLETE_CALENDAR;
	configData.agendaShowCompleteTodo = CONFIG_AGENDA_SHOW_COMPLETE_TODO;
	configData.agendaAutoExpandCalendar = CONFIG_AGENDA_AUTOEXPAND_CALENDAR;
	configData.agendaAutoExpandTodo = CONFIG_AGENDA_AUTOEXPAND_TODO;
	configData.defaultView = CONFIG_FN_DEFAULT_VIEW;

	// Config cannot be loaded now because we need the full path of the application and
	// this is only available after the application is constructed. Therefore, config
	// loading happens in the App class's ConstructL() member
}

CQikAppUi* CSMan2Document::CreateAppUiL()
{
	return new(ELeave) CSMan2AppUi(&configData);
}

/*************************************************************
*
* AppUi object
*
**************************************************************/

CSMan2AppUi::~CSMan2AppUi()
{
	DeregisterView(*iTaskView);
	DeregisterView(*iFileView);
	DeregisterView(*iFlipCloseView);
	DeregisterView(*iBJackView);
	DeregisterView(*iSysInfoView);
	DeregisterView(*iDBView);
	DeregisterView(*iAgendaView);
	
	delete iTaskView;
	delete iFileView;
	delete iFlipCloseView;
	delete iBJackView;
	delete iSysInfoView;
	delete iDBView;
	delete iAgendaView;
	
	delete smanFileMan;
	delete startUidList;
	delete historyUidList;
}

void CSMan2AppUi::HandleForegroundEventL(TBool aForeground)
{
	CQikAppUi::HandleForegroundEventL(aForeground);
	if (aForeground)
	{
		appInForeground = ETrue;
		
		// Flip open mode
		if (CEikonEnv::Static()->ScreenDevice()->CurrentScreenMode() == 0)
		{
			TVwsViewId defaultView;
			if (GetDefaultViewId(defaultView) == KErrNone)
			{
				if (defaultView.iViewUid == KUidTaskView)
					iTaskView->refreshTaskList(TODO_REFRESH);
			}
		}
	
		// Re-activate marquee if we are in file view
		if (!(iFileView->currentPathScroller->IsActive()))
			iFileView->currentPathScroller->startMarquee(iFileView->labelCurrentFolder, &(iFileView->currentPath));
	}
	else
	{
		appInForeground = EFalse;
		
		/*
		if (CEikonEnv::Static()->ScreenDevice()->CurrentScreenMode() == 1)
		{
			iFlipCloseView->popupActive = EFalse;
		}
		*/

		// Stop the marquee to conserve battery power
		if (iFileView->currentPathScroller->IsActive())
			iFileView->currentPathScroller->Cancel();
	}
}

void CSMan2AppUi::switchView(TInt viewId)
{		
	// Save battery
	if (iFileView->currentPathScroller->IsActive())
		iFileView->currentPathScroller->Cancel();

	if (viewId == EViewAgenda)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_AGENDA_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, iOldMenuBarYPos));
		
		SetToolbarL(R_TOOLBAR_AGENDA);
		SetDefaultViewL(*iAgendaView);
	}
	if (viewId == EViewTask)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_MEMORY_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, iOldMenuBarYPos));

		RemoveToolbarL();
		SetDefaultViewL(*iTaskView);
	}
	else if (viewId == EViewBJack)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_BLUEJACK_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, iOldMenuBarYPos));

		SetToolbarL(R_TOOLBAR_BLUEJACK);
		SetDefaultViewL(*iBJackView);

		iBJackView->oScanner->SetToolbarButtonState();
	}
	else if (viewId == EViewDB)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_DB_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, iOldMenuBarYPos));

		if (iDBView->currentDBView == CSMan2DBView::DBView_List)
			SetToolbarL(R_TOOLBAR_DB_LIST);
		else
			SetToolbarL(R_TOOLBAR_DB_SINGLE);
			
		//SetDefaultViewL(*iDBView);
	}
	else if (viewId == EViewSysInfo)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_SYSINFO_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, iOldMenuBarYPos));

		RemoveToolbarL();
		//SetDefaultViewL(*iSysInfoView);
	}
	else if (viewId == EViewFile)
	{
		if (!(iFileView->currentPathScroller->IsActive()))
			iFileView->currentPathScroller->startMarquee(iFileView->labelCurrentFolder, &(iFileView->currentPath));

		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_FILE_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, iOldMenuBarYPos));
			
		SetToolbarL(R_TOOLBAR_FILEMAN);
		SetDefaultViewL(*iFileView);
	}
	else if (viewId == EViewFlipClosed)
	{
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(EFalse);
		
/* DEBUG TEST
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->ChangeMenuBarL(0, R_FC_BLUEJACK_MENUBAR, EFalse);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
		CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetPosition(TPoint(0, 0));
*/

		RemoveToolbarL();
	}
}

void CSMan2AppUi::updateBJackIcon(TBool draw)
{
	// To be sure we get this right, update ALL views.
	iTaskView->drawBJackIcon(draw);
	iFileView->drawBJackIcon(draw);
	iFlipCloseView->drawBJackIcon(draw);
	iBJackView->drawBJackIcon(draw);
	iDBView->drawBJackIcon(draw);
	iSysInfoView->drawBJackIcon(draw);
	iAgendaView->drawBJackIcon(draw);	
}

void CSMan2AppUi::ConstructL()
{
	BaseConstructL();

	appInForeground = EFalse;
	
	// Note: You need to load this first because some of the views created rely on the
	// loaded values for initialization
	static_cast<CSMan2Document*>(Document())->configFileName.Copy(CEikonEnv::Static()->EikAppUi()->Application()->AppFullName());
	static_cast<CSMan2Document*>(Document())->configFileName.SetLength(static_cast<CSMan2Document*>(Document())->configFileName.Length() - 3);
	static_cast<CSMan2Document*>(Document())->configFileName.Append(_L("cfg"));
	
	// Load config and initialize vars based on config
	static_cast<CSMan2Document*>(Document())->loadConfig();
	//CEikonEnv::Static()->InfoMsg(R_TBUF_ERRLOADCONFIG);

	oldScreenMode = iCoeEnv->ScreenDevice()->CurrentScreenMode();
	bjackIconLabelCurrentImage = 0;

	iTaskView = new (ELeave) CSMan2TaskListView(configData);
	iTaskView->ConstructL(ClientRect());

	iFlipCloseView = new (ELeave) CSMan2FlipCloseView(configData);
	iFlipCloseView->ConstructL(ClientRect());

	iFileView = new (ELeave) CSMan2FileListView(configData);
	iFileView->ConstructL(ClientRect());

	iBJackView = new (ELeave) CSMan2BJackView(configData);
	iBJackView->ConstructL(ClientRect());

	iSysInfoView = new (ELeave) CSMan2SysInfoView(configData);
	iSysInfoView->ConstructL(ClientRect());

	iDBView = new (ELeave) CSMan2DBView(configData);
	iDBView->ConstructL(ClientRect());
	
	iAgendaView = new (ELeave) CSMan2AgendaView(configData);
	iAgendaView->ConstructL(ClientRect());
	
	RegisterViewL(*iFileView);
	RegisterViewL(*iFlipCloseView);
	RegisterViewL(*iTaskView);
	RegisterViewL(*iBJackView);
	RegisterViewL(*iSysInfoView);
	RegisterViewL(*iDBView);
	RegisterViewL(*iAgendaView);
	
	smanFileMan = new (ELeave) CSManFileMan(configData);
	startUidList = new (ELeave) CArrayFixFlat<TUid>(3);
	historyUidList = new (ELeave) CArrayFixFlat<TUid>(NUM_LASTRUN_HISTORY);
	historyUidList->AppendL(KNullUid);
	historyUidList->AppendL(KNullUid);
	historyUidList->AppendL(KNullUid);

	// This variable shouldn't be initialized here. It gets initialized in base class of each view
	//iViewId = configData->defaultView;

	// This is needed because in FC mode we reposition the menubar to the top of the screen
	iOldMenuBarYPos = CEikonEnv::Static()->AppUiFactory()->MenuBar()->Position().iY;
	
	switch (configData->defaultView)
	{
		case EViewTask : 
			{
				SetDefaultViewL(*iTaskView);
				iTaskView->refreshTaskList(TODO_REFRESH);
				break;
			}
		case EViewFile :
			{
				SetDefaultViewL(*iFileView);
				iFileView->fillListBox(_L("C:\\"));
				break;
			}
		case EViewBJack :
			{
				SetDefaultViewL(*iBJackView);
				break;
			}
		case EViewDB :
			{
				SetDefaultViewL(*iDBView);
				break;
			}
		case EViewSysInfo :
			{
				SetDefaultViewL(*iSysInfoView);
				// The ViewActivatedL member of this view already calls refreshSysInfo
				//iSysInfoView->refreshSysInfo();
				break;
			}
		case EViewAgenda :
			{
				SetDefaultViewL(*iAgendaView);
				iAgendaView->ReadAgenda(EFalse);
				break;
			}
		// This should never be called unless the config file is corrupted :(
		default:
			{
				SetDefaultViewL(*iTaskView);
				iTaskView->refreshTaskList(TODO_REFRESH);
				break;
			}
	};
	if (oldScreenMode == 1)
		CEikonEnv::Static()->InfoMsg(R_TBUF_SMANLOADED);
}

LOCAL_C void CloseLibrary(TAny* aLibrary)
{
	((RLibrary*)aLibrary)->Close();
}

void CSMan2AppUi::launchControlPanel(TPtrC appletName)
{
	RLibrary control;
	CleanupStack::PushL(TCleanupItem(CloseLibrary, &control));
	User::LeaveIfError(control.Load(appletName));
	(*(TLoadDll)control.Lookup(1))(appletName);
	CleanupStack::PopAndDestroy(); // control
}

void CSMan2AppUi::doFileMan(TInt op)
{
	smanFileMan->fileList = iFileView->clipBoard;
	smanFileMan->fileManOperation = iFileView->clipBoardOperation;
	smanFileMan->targetPath.Copy(iFileView->currentPath);
	smanFileMan->currentOperation = op;
	smanFileMan->doWork();
}

void CSMan2AppUi::SetZoom(TInt zoomLevel)
{
	if (viewId == EViewTask)
		iTaskView->SetZoomLevel(zoomLevel);
	else if (viewId == EViewBJack)
		iBJackView->SetZoomLevel(zoomLevel);
	else if (viewId == EViewDB)
		iDBView->SetZoomLevel(zoomLevel);
	else if (viewId == EViewFile)
		iFileView->SetZoomLevel(zoomLevel);
	else if (viewId == EViewSysInfo)
		iSysInfoView->SetZoomLevel(zoomLevel);
	else if (viewId == EViewAgenda)
		iAgendaView->SetZoomLevel(zoomLevel);
	static_cast<CSMan2Document*>(Document())->saveConfig();
}

void CSMan2AppUi::doProcessCommand(TInt aCommand)
{
	ProcessCommandL(aCommand);
}

void CSMan2AppUi::doCancelMenu(void)
{
	doProcessCommand(EEikCmdCanceled);
	//iFlipCloseView->popupActive = EFalse;
}

void CSMan2AppUi::FileManGetFile(TPtrC *entryType, TPtrC *entryName)
{
	TPtrC tempText;
	
	tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(iFileView->cFileListBox->View()->CurrentItemIndex()));
	TextUtils::ColumnText(*entryName, 0, &tempText);
	TextUtils::ColumnText(*entryType, 1, &tempText);
}

TBool CSMan2AppUi::PromptAppFixOrder(TInt aPromptResourceId)
{
	if (!ConeUtils::FileExists(KAppConfigFileBackup))
		return ETrue;
	
	TBool retVal = EFalse;
	
	HBufC *title = CEikonEnv::Static()->AllocReadResourceL(R_TBUF_DB_CONFIRM_CANCEL_TITLE);
	HBufC *warning = CEikonEnv::Static()->AllocReadResourceL(aPromptResourceId);
	
	TTime backupFileDateTime;
	CEikonEnv::Static()->FsSession().Modified(KAppConfigFileBackup, backupFileDateTime);
	TBuf<30> backupFileDateTimeText;
	backupFileDateTime.FormatL(backupFileDateTimeText, _L("%/0%1%/1%2%/2%3%/3 %-B%:0%J%:1%T%:2%S%:3%+B  "));
	warning = warning->ReAllocL(warning->Length() + backupFileDateTimeText.Length());
	warning->Des().Append(backupFileDateTimeText);
	
	if (CEikonEnv::Static()->QueryWinL(*title, *warning))
		retVal = ETrue;

	delete title;
	delete warning;
	return retVal;
}

void CSMan2AppUi::HandleCommandL(TInt aCommand)
{
	TInt i;
	TPtrC tempText, entryType, entryName;

	/*************************************************************
	*
	* Zoom command appears everywhere
	*
	**************************************************************/
	if (aCommand == cmdZoom)
	{
		TInt zoomState = 0;
		
		// This is faster than calling APIs....
		if (viewId == EViewTask)
			zoomState = configData->zoomMain;
		else if (viewId == EViewBJack)
			zoomState = configData->zoomBJack;
		else if (viewId == EViewDB)
			zoomState = configData->zoomDB;
		else if (viewId == EViewFile)
			zoomState = configData->zoomFile;
		else if (viewId == EViewSysInfo)
			zoomState = configData->zoomSysInfo;
		else if (viewId == EViewAgenda)
			zoomState = configData->zoomAgenda;
		CQikZoomDialog::RunDlgLD((zoomState / ZOOM_INCREMENT) + EQikCmdZoomLevel1 - 1);
	}
	else if ((aCommand == EQikCmdZoomLevel1) || (aCommand == EQikCmdZoomLevel2) || (aCommand == EQikCmdZoomLevel3))
	{
		SetZoom(ZOOM_INCREMENT * (aCommand - EQikCmdZoomLevel1 + 1));
	}
	/*************************************************************
	*
	* Control panel shortcuts. Thanks to Phil Spencer of Symbian
	*
	**************************************************************/
	if (aCommand == cmdCPIrda)
	{
		launchControlPanel(_L("Z:\\System\\Controls\\irdase.ctl"));
	}
	else if (aCommand == cmdCPBluetooth)
	{
		launchControlPanel(_L("Z:\\System\\Controls\\btconfig.ctl"));
	}
	/*************************************************************
	*
	* FN menu commands. Exists in all views.
	*
	**************************************************************/
	else if (aCommand == cmdSwitchMemory)
	{
		if (iTaskView->firstTime)
		{
			iTaskView->refreshTaskList(TODO_REFRESH);
			iTaskView->firstTime = EFalse;
		}
		ActivateViewL(TVwsViewId(KUidSMan2App, KUidTaskView));
	}
	else if (aCommand == cmdSwitchFile)
	{
		if (iFileView->firstTime)
		{
			iFileView->fillListBox(_L("C:\\"));
			iFileView->firstTime = EFalse;
		}
		ActivateViewL(TVwsViewId(KUidSMan2App, KUidFileView));
	}
	else if (aCommand == cmdSwitchBJack)
	{
		ActivateViewL(TVwsViewId(KUidSMan2App, KUidBJackView));
	}
	else if (aCommand == cmdSwitchDB)
	{
		ActivateViewL(TVwsViewId(KUidSMan2App, KUidDBView));
	}
	else if (aCommand == cmdSwitchSysInfo)
	{
		ActivateViewL(TVwsViewId(KUidSMan2App, KUidSysInfoView));
	}
	else if (aCommand == cmdSwitchAgenda)
	{
		if (iAgendaView->firstTime)
		{
			iAgendaView->ReadAgenda(ETrue);
			iAgendaView->firstTime = EFalse;
		}
		ActivateViewL(TVwsViewId(KUidSMan2App, KUidAgendaView));
	}
	else if (aCommand == cmdSwitchOptions)
	{
		CFnMenuToggle *switchDialog = new (ELeave) CFnMenuToggle(&(configData->fnActivatedMenus), &(configData->defaultView));
		if (switchDialog->ExecuteLD(R_DIALOG_FN_OPTIONS) == EEikBidOk)
			static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdControlPanel)
	{
		launchApp(KUidControlPanel, _L(""), EFalse);
	}
	/*************************************************************
	*
	* Commands for Agenda View.
	*
	**************************************************************/
	else if (aCommand == cmdAgendaViewOtherDate)
	{
		iAgendaView->ViewOtherDate();
	}
	else if (aCommand == cmdAgendaRead)
	{
		iAgendaView->ReadAgenda(ETrue);
	}
	else if (aCommand == cmdAgendaCompleteCalendar)
	{
		configData->agendaShowCompleteCalendar = !configData->agendaShowCompleteCalendar;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iAgendaView->ReadAgenda(ETrue);
	}
	else if (aCommand == cmdAgendaCompleteTodo)
	{
		configData->agendaShowCompleteTodo = !configData->agendaShowCompleteTodo;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iAgendaView->ReadAgenda(ETrue);
	}
	else if (aCommand == cmdAgendaAutoExpandCalendar)
	{
		configData->agendaAutoExpandCalendar = !configData->agendaAutoExpandCalendar;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdAgendaAutoExpandTodo)
	{
		configData->agendaAutoExpandTodo = !configData->agendaAutoExpandTodo;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	/*************************************************************
	*
	* Commands for Sysinfo View
	*
	**************************************************************/
	else if (aCommand == cmdSysInfoRefresh)
	{
		iSysInfoView->refreshSysInfo();
	}
	/*************************************************************
	*
	* Commands for Database View
	*
	**************************************************************/
	else if (aCommand == cmdDBCloseDB)
	{
		if (iDBView->dbIsOpened)
			iDBView->doCloseDB(ETrue);
	}
	else if (aCommand == cmdDBSortAsc)
	{
		configData->dbSortAscending = ETrue;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iDBView->ShowRecordsList(ETrue, EFalse);
	}
	else if (aCommand == cmdDBSortDes)
	{
		configData->dbSortAscending = EFalse;
		static_cast<CSMan2Document*>(Document())->saveConfig();		
		iDBView->ShowRecordsList(ETrue, EFalse);
	}
	else if (aCommand == cmdDBReInit)
	{
		iDBView->doReInitDB();
	}
	else if (aCommand == cmdDBChangePassword)
	{
		iDBView->doChangePassword();
	}
	else if (aCommand == cmdDBReadDB)
	{
		iDBView->doReadDB();
	}
	else if (aCommand == cmdDBDo)
	{
		LaunchPopupMenuL(R_DB_DOBUTTON, TPoint(iDBView->Size().iWidth, iDBView->PositionRelativeToScreen().iY + iDBView->Size().iHeight - 2), EPopupTargetBottomRight);
	}
	else if (aCommand == cmdDBBack)
	{
		if ((iDBView->currentDBOp == CSMan2DBView::DBOp_Editing) || (iDBView->currentDBOp == CSMan2DBView::DBOp_Adding))
		{
			HBufC* dataBuffer = CEikonEnv::Static()->AllocReadResourceL(R_TBUF_DB_CONFIRM_CANCEL_TITLE);
			HBufC* dataBuffer2 = CEikonEnv::Static()->AllocReadResourceL(R_TBUF_DB_CONFIRM_CANCEL);
			if (CEikonEnv::Static()->QueryWinL(*dataBuffer, *dataBuffer2))
				iDBView->ShowRecordsList(EFalse, ETrue);
			delete dataBuffer;
			delete dataBuffer2;
		}
		else
			iDBView->ShowRecordsList(EFalse, ETrue);
	}
	else if (aCommand == cmdDBSave)
	{
		if (iDBView->SaveRecord(iDBView->currentRecordIndex))
			iDBView->ShowRecordsList(ETrue, ETrue);
		else
			iEikonEnv->InfoMsg(R_TBUF_DB_ERR_SAVE);
	}
	else if (aCommand == cmdDBView)
	{
		iDBView->doDisplayOneRecord(ETrue);
	}
	else if (aCommand == cmdDBDel)
	{
		if (!iDBView->doDelete())
			iEikonEnv->InfoMsg(R_TBUF_DB_ERR_MASS_DELETE);
		else
			iDBView->ShowRecordsList(ETrue, EFalse);
	}
	else if (aCommand == cmdDBAdd)
	{
		iDBView->ShowRecordsSingle(0, ETrue);
	}
	else if (aCommand == cmdDBEdit)
	{
		iDBView->doDisplayOneRecord(EFalse);
	}
	else if (aCommand == cmdDBCompact)
	{
		if (!iDBView->doCompact())
			iEikonEnv->InfoMsg(R_TBUF_DB_ERR_COMPACT);
		else
			iEikonEnv->InfoMsg(R_TBUF_MISC_DONE);
	}
	else if (aCommand == cmdDBUpdateStats)
	{
		if (!iDBView->doUpdateStats())
			iEikonEnv->InfoMsg(R_TBUF_DB_ERR_UPDATESTATS);
		else
			iEikonEnv->InfoMsg(R_TBUF_MISC_DONE);
	}
	/*************************************************************
	*
	* Commands for File View.
	*
	**************************************************************/
	else if (aCommand == cmdDoButton)
	{
		LaunchPopupMenuL(R_FILEMAN_DOBUTTON, TPoint(iFileView->Size().iWidth, iFileView->cFileListBox->PositionRelativeToScreen().iY + iFileView->cFileListBox->Size().iHeight + 6), EPopupTargetBottomRight);
	}
	else if (aCommand == cmdFileMan_IrDA)
	{
		configData->beamProtocol = CBlueBeam::protocolIrDA;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iFileView->cFileListBox->ItemDrawer()->SetHighlightedBackColor(TRgb(255, 0, 0));		
		iFileView->cFileListBox->DrawDeferred();
	}
	else if (aCommand == cmdFileMan_Bluetooth)
	{
		configData->beamProtocol = CBlueBeam::protocolBluetooth;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iFileView->cFileListBox->ItemDrawer()->SetHighlightedBackColor(TRgb(0, 0, 255));
		iFileView->cFileListBox->DrawDeferred();
	}
	else if (aCommand == cmdFileManReload)
	{
		iFileView->fillListBox(iFileView->currentPath);
	}
	else if ((aCommand == cmdFileManOpen) || (aCommand == cmdFileManOpenOverride))
	{
		FileManGetFile(&entryType, &entryName);
		if ((entryType.Compare(_L("<drv>")) != 0) && (entryName.Compare(_L("..")) != 0) && (entryType.Compare(_L("<dir>")) != 0))
		{
			TFileName fileName;

			fileName.Copy(iFileView->currentPath);
			fileName.Append(entryName);
			fileName.UpperCase();

			TParsePtr tempName(fileName);
			if (tempName.Ext() == _L(".APP"))
			{
				if ((configData->autoRunApp) || (aCommand == cmdFileManOpenOverride))
					FindAndLaunchAppNoUid(&(tempName.FullName()));
			}
			else if (tempName.Ext() == _L(".EXE"))
			{
				if ((configData->autoRunApp) || (aCommand == cmdFileManOpenOverride))
					EikDll::StartExeL(tempName.FullName());
			}
/*			
			else if (tempName.Ext() == _L(".CLASS"))
			{			
				CApaCommandLine* cmdLine=CApaCommandLine::NewLC();
				cmdLine->SetLibraryNameL(fileName);
				cmdLine->SetCommandL(EApaCommandOpen);
				RApaLsSession ls;
				User::LeaveIfError(ls.Connect());
				CleanupClosePushL(ls);
				User::LeaveIfError(ls.StartApp(*cmdLine));
				CleanupStack::PopAndDestroy(2); // ls and cmdLine
			}
*/
			else if ((configData->autoOpenFile) || (aCommand == cmdFileManOpenOverride))
			{
				QikFileUtils::StartAppL(fileName);
			}
		}
		else
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_NOTAFILE);
/*
			RApaLsSession apaLsSession;
			TUid appUid;
			TDataType dataType;

			if (apaLsSession.Connect() == KErrNone)
			{
				CleanupClosePushL(apaLsSession);
				if (apaLsSession.AppForDocument(fileName, appUid, dataType) == KErrNone)
				{
					CleanupStack::PopAndDestroy();
					if (appUid == TUid::Null())
						User::InfoPrint(_L("No recognizer"));
					else
					{
						launchApp(appUid, fileName, EFalse);
					}
				}
				else
				{
					User::InfoPrint(_L("Call err: AppForDocument()"));
					CleanupStack::PopAndDestroy();
				}
			}
			else
				User::InfoPrint(_L("Connect err: RApaLsSession"));
		}
		else
			User::InfoPrint(_L("Invalid highlight"));
*/
	}
	else if (aCommand == cmdFileManToggleHiddenView)
	{
		configData->showHiddenFiles = !configData->showHiddenFiles;
		iFileView->fillListBox(iFileView->currentPath);
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdFileManToggleRunApp)
	{
		configData->autoRunApp = !configData->autoRunApp;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdFileManCRC)
	{
		FileManGetFile(&entryType, &entryName);
		if ((entryType.Compare(_L("<drv>")) != 0) && (entryName.Compare(_L("..")) != 0) && (entryType.Compare(_L("<dir>")) != 0))
		{
			TFileName fileName;

			fileName.Copy(iFileView->currentPath);
			fileName.Append(entryName);
			iFileView->crcCalculator->doCRC(fileName);
		}
		else
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_CRCERROR);
	}
	else if (aCommand == cmdFileManToggleOpenFile)
	{
		configData->autoOpenFile = !configData->autoOpenFile;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdFileManProperties)
	{		
		FileManGetFile(&entryType, &entryName);
		if ((entryType.Compare(_L("<drv>")) != 0) && (entryName.Compare(_L("..")) != 0))
		{
			CFileManAttrDialog *attDialog = new (ELeave) CFileManAttrDialog();
			attDialog->theFile.Copy(iFileView->currentPath);
			attDialog->theFile.Append(entryName);
			TInt retVal = attDialog->ExecuteLD(R_DIALOG_FILEMANATT);
			if (retVal == EEikBidOk)
				iFileView->fillListBox(iFileView->currentPath);
		}
		else 
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_NOTAFILE);
	}
	else if (aCommand == cmdFileManAll)
	{
		for (i = 0; i < iFileView->cFileListBox->Model()->ItemTextArray()->MdcaCount(); i++)
		{
			tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(i));
			TextUtils::ColumnText(entryType, 1, &tempText);
			TextUtils::ColumnText(entryName, 0, &tempText);
			if ((entryType.Compare(_L("<drv>")) != 0) && (entryName.Compare(_L("..")) != 0))
				iFileView->cFileListBox->View()->SelectItemL(i);
		}
	}
	else if (aCommand == cmdFileManNone)
	{
		for (i = 0; i < iFileView->cFileListBox->Model()->ItemTextArray()->MdcaCount(); i++)
		{
			tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint(i));
			TextUtils::ColumnText(entryType, 1, &tempText);
			iFileView->cFileListBox->View()->DeselectItem(i);
		}
	}
	else if (aCommand == cmdFileManBeam)
	{
		if (!iBJackView->oScanner->IsActive())
		{
			if (iFileView->blueBeam->IsActive())
				iEikonEnv->InfoMsg(R_TBUF_BEAMER_BUSY);
			else
			{
				TBool hasFiles = EFalse;
				
				for (i = 0; i < iFileView->cFileListBox->SelectionIndexes()->Count(); i++)
				{
					tempText.Set(iFileView->cFileListBox->Model()->ItemTextArray()->MdcaPoint((iFileView->cFileListBox->SelectionIndexes()->At(i))));
					TextUtils::ColumnText(entryType, 1, &tempText);
					if (entryType.Compare(_L("<dir>")) != 0)
					{
						hasFiles = ETrue;
						break;
					}
				}
				
				if (hasFiles)
				{
					iFileView->blueBeam->logHelper->logDialog = NULL;
					iFileView->blueBeam->logHelper->logTarget = CLogger::logToDialog;
					iFileView->blueBeam->beamerProtocol = configData->beamProtocol;
					iFileView->blueBeam->startBeam();
				}
				else
					iEikonEnv->InfoMsg(R_TBUF_FILEMAN_NOFILES);
			}
		}
		else
			iEikonEnv->InfoMsg(R_TBUF_BJACKBUSY);
	}
	else if ((aCommand == cmdFileManCopy) || (aCommand == cmdFileManCut))
	{
		TInt op = (aCommand == cmdFileManCopy) ? CSManFileMan::operationCopy : CSManFileMan::operationCut;

		if ((i = iFileView->populateClipboard(op)) > 0)
		{
			if (op == CSManFileMan::operationCut)
				iEikonEnv->InfoMsg(R_TBUF_FILEMAN_CUT, i);
			else if (op == CSManFileMan::operationCopy)
				iEikonEnv->InfoMsg(R_TBUF_FILEMAN_COPY, i);
		}
		else
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_CLIPBFLUSH, i);
	}
	else if (aCommand == cmdFileManPaste)
	{
		if (iFileView->clipBoard->Count() > 0)
			doFileMan(CSManFileMan::isPaste);
		else
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_CLIPEMPTY);
	}
	else if (aCommand == cmdFileManDelete)
	{
		if (iFileView->populateClipboard(-1) > 0)
			doFileMan(CSManFileMan::isDelete);
		else
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_NOFILES);
	}
	else if (aCommand == cmdFileManRename)
	{
		if (iFileView->populateClipboard(-1) > 0)
			doFileMan(CSManFileMan::isRename);
		else
			iEikonEnv->InfoMsg(R_TBUF_FILEMAN_NOFILES);		
	}	
	else if (aCommand == cmdFileManNewFolder)
	{
		CFileManNewDirDialog *newDir = new (ELeave) CFileManNewDirDialog;
		newDir->currentPath.Copy(iFileView->currentPath);
		newDir->ExecuteLD(R_DIALOG_FILEMANNEWFOLDER);
		iFileView->fillListBox(iFileView->currentPath);
	}	
	else if ((aCommand >= cmdFileMan_Drive_A) && (aCommand <= cmdFileMan_Drive_Z))
	{
		TBuf<3> drvPath;
		
		drvPath.Format(_L("%c:\\"), aCommand - cmdFileMan_Drive_A + 0x41);
		iFileView->fillListBox(drvPath);
	}
	/*************************************************************
	*
	* Commands for Utilities
	*
	**************************************************************/
	else if (aCommand == cmdUtilsAgendaFixSynchTimeZone)
	{
		iAgendaView->FixSynchTimeZone();
	}
	else if (aCommand == cmdUtilsFixApplist)
	{
		TApaAppInfo dummy;
		
		static_cast<CSMan2Document*>(Document())->saveConfig();
		FindApp(KUidControlPanel, &dummy);
		iEikonEnv->InfoMsg(R_TBUF_APPLISTFIXED);
	}
	else if (aCommand == cmdUtilsCompContacts)
	{
		CEikonEnv::Static()->BusyMsgL(R_TBUF_MISC_BUSY);	
		CContactDatabase* cdb = CContactDatabase::OpenL();
		TRAPD(error, cdb->CompactL());
		delete cdb;
		CEikonEnv::Static()->BusyMsgCancel();
		if (error != KErrNone)
			iEikonEnv->InfoMsg(R_TBUF_UTIL_ERRCOMPACTCONTACTS);
		else
			iEikonEnv->InfoMsg(R_TBUF_MISC_DONE);
	}
	else if (aCommand == cmdUtilsFixAppOrderReset)
	{
		HBufC *title = CEikonEnv::Static()->AllocReadResourceL(R_TBUF_DB_CONFIRM_CANCEL_TITLE);
		HBufC *warning = CEikonEnv::Static()->AllocReadResourceL(R_TBUF_UTILS_FIXAPPORDER_CONFIRMRESET);
		if (CEikonEnv::Static()->QueryWinL(*title, *warning))
		{
			TBuf<30> appName;
			appName.Copy(KAppTaskName);
			EikFileUtils::DeleteFile(KAppConfigFile);
			iTaskView->gentleEndTask(appName);
			iEikonEnv->InfoMsg(R_TBUF_UTILS_FIXAPPORDER_WAITAWHILE);
		}
		delete title;
		delete warning;
	}
	else if (aCommand == cmdUtilsFixAppOrderRestore)
	{
		if (PromptAppFixOrder(R_TBUF_UTILS_FIXAPPORDER_CONFIRMRESTORE))
		{
			if (EikFileUtils::CopyFile(KAppConfigFileBackup, KAppConfigFile, CFileMan::EOverWrite) == KErrNone)
			{
				TBuf<30> appName;
				appName.Copy(KAppTaskName);
				iTaskView->gentleEndTask(appName);
				iEikonEnv->InfoMsg(R_TBUF_UTILS_FIXAPPORDER_WAITAWHILE);
			}
			else
				iEikonEnv->InfoMsg(R_TBUF_UTILS_FIXAPPORDER_FAILED);
		}
	}
	else if (aCommand == cmdUtilsFixAppOrderBackup)
	{
		if (PromptAppFixOrder(R_TBUF_UTILS_FIXAPPORDER_CONFIRMBACKUP))
		{
			if (EikFileUtils::CopyFile(KAppConfigFile, KAppConfigFileBackup, CFileMan::EOverWrite) == KErrNone)
				iEikonEnv->InfoMsg(R_TBUF_MISC_DONE);
			else
				iEikonEnv->InfoMsg(R_TBUF_UTILS_FIXAPPORDER_BACKUPFAIL);
		}
	}
	/*************************************************************
	*
	* Commands for Task View
	*
	**************************************************************/
	else if (aCommand == cmdShortcuts)
	{
		CArrayFixFlat<TUid> *tempUid;

		tempUid = new (ELeave) CArrayFixFlat<TUid>(NUM_SHORTCUTS);
		for (i = 0; i < NUM_SHORTCUTS; i++)
		{
			tempUid->AppendL(configData->shortCutUid[i]);
		}
		CShortcutDialog* shortcutDialog = new (ELeave) CShortcutDialog(tempUid);
		if (shortcutDialog->ExecuteLD(R_DIALOG_SHORTCUT) == EEikBidOk)
		{
			for (i = 0; i < NUM_SHORTCUTS; i++)
				configData->shortCutUid[i] = tempUid->At(i);
			static_cast<CSMan2Document*>(Document())->saveConfig();
		}
		delete tempUid;
	}
	else if ((aCommand >= cmdShort1) && (aCommand <= cmdShort7))
	{
		for (i = 0; i < NUM_SHORTCUTS; i++)
		{
			if (aCommand == (cmdShort1 + i))
			{
				launchApp(configData->shortCutUid[i], _L(""), EFalse);
				break;
			}
		}
	}
	else if ((aCommand >= cmdLastRunBase) && ((aCommand < cmdLastRunBase + NUM_LASTRUN_HISTORY)))
	{
		launchApp(historyUidList->At(aCommand - cmdLastRunBase), _L(""), ETrue);
	}
	else if (aCommand >= cmdRunAppBase)
	{
		TBool addToHistory = ETrue;
		// Check if the entry is in the shortcut list. If so, do not add it to history
		for (int i = 0; i < NUM_SHORTCUTS; i++)
			if (configData->shortCutUid[i] == startUidList->At(aCommand - cmdRunAppBase))
			{
				addToHistory = EFalse;
				break;
			}
		
		launchApp(startUidList->At(aCommand - cmdRunAppBase), _L(""), addToHistory);
		startUidList->Reset();
	}
	else if (aCommand == cmdPersist)
	{
		configData->persist = !configData->persist;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdAutostartOthers)
	{
		CAutostartDialog* autoStartDialog = new (ELeave) CAutostartDialog();
		autoStartDialog->ExecuteLD(R_DIALOG_AUTOSTART);
	}
	else if (aCommand == cmdAutostartSMan)
	{
		CAutostartDialog* autoStartDialog = new (ELeave) CAutostartDialog();
		autoStartDialog->loadBootFile();		
		if (autoStartDialog->autoStartConfig.autoStartUid[MAX_AUTOSTART - 1] != KUidSMan2App)
		{
			// Ensure we get upper case stuff only!
			autoStartDialog->autoStartConfig.autoStartApp[MAX_AUTOSTART - 1].Copy(CEikonEnv::Static()->EikAppUi()->Application()->AppFullName().Left(2));
			autoStartDialog->autoStartConfig.autoStartApp[MAX_AUTOSTART - 1].Append(_L("\\SYSTEM\\APPS\\SMAN\\SMAN.APP"));
			autoStartDialog->autoStartConfig.autoStartUid[MAX_AUTOSTART - 1] = KUidSMan2App;
		}
		else
		{
			autoStartDialog->autoStartConfig.autoStartApp[MAX_AUTOSTART - 1].Copy(_L(""));
			autoStartDialog->autoStartConfig.autoStartUid[MAX_AUTOSTART - 1] = KNullUid;
		}
		if (autoStartDialog->saveBootFile() != 0)
			iEikonEnv->InfoMsg(R_TBUF_ERRSAVECONFIG);
		delete autoStartDialog;
	}
	else if (aCommand == cmdLogFlags)
	{
		CLogLevelDialog* logLevelDialog = new (ELeave) CLogLevelDialog(&(configData->logFlags));
		if (logLevelDialog->ExecuteLD(R_DIALOG_LOGLEVELS) == EEikBidOk)
			static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdShowHiddenTask)
	{
		configData->showHiddenTasks = !configData->showHiddenTasks;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else if (aCommand == cmdNoDevicePause)
	{
		CEikDialog* noDevicePauseDialog = new(ELeave) CBluejackTimerDialog(&(configData->noDevicePause), &(configData->noDeviceAttempts), &(configData->decayTime), &(configData->obexTimeout));
		if (noDevicePauseDialog->ExecuteLD(R_DIALOG_NODEVICEPAUSE) == EEikBidOk)
			static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdForceEndTask)
	{
		configData->forceEndTask = !configData->forceEndTask;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdFlipFlush)
	{
		if (!configData->flipFlush)
		{
			if (iTaskView->snapshotTasks->Count() <= 0)
				iEikonEnv->InfoMsg(R_TBUF_NOSNAPSHOT);
			else
				configData->flipFlush = !configData->flipFlush;
		}
		else
			configData->flipFlush = !configData->flipFlush;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdEndTask)
	{
		if (iTaskView->cTaskListBox->SelectionIndexes()->Count() > 0)
		{
			iTaskView->refreshTaskList(TODO_TERMINATE);			
		}
		else
			iEikonEnv->InfoMsg(R_TBUF_NOMARKEDTASKS);
	}
	else if (aCommand == cmdBluejackAutoSave)
	{
		configData->autoSave = !configData->autoSave;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdFlushOut)
	{
		flushTasks();
	}
	else if (aCommand == cmdSnapshot)
	{
		CEikDialog* snapshotDialog = new(ELeave) CEikDialog;
		if (snapshotDialog->ExecuteLD(R_DIALOG_SNAPSHOTCONFIRM) == EEikBidYes)
			updateSnapShot();
	}
	else if (aCommand == cmdHotkey)
	{
		CEikDialog* dialog = new(ELeave) CHotkeyDialog(&(configData->iHotkeyDialogResult));
		if (dialog->ExecuteLD(R_DIALOG_HOTKEY) == EEikBidOk)
		{
			iTaskView->ToggleHotKey();
			static_cast<CSMan2Document*>(Document())->saveConfig();
		}
	}
	else if ((aCommand == cmdQuit) || (aCommand == EEikCmdExit))
	{
		if ((aCommand == EEikCmdExit) && (configData->persist))
		{
			// Do nothing
		}
		else
		{
			if (iBJackView->oScanner->IsActive())
			{
				if (iBJackView->oScanner->scannerStatus == CScanner::isShutdown)
					iEikonEnv->InfoMsg(R_TBUF_SCANNERSTOPPING);
				else
					iEikonEnv->InfoMsg(R_TBUF_STOPSCANNER);
			}
			else if (iFileView->blueBeam->IsActive())
				iEikonEnv->InfoMsg(R_TBUF_STOPBEAM);
			else
				Exit();
		}
	}
	else if (aCommand == cmdCompressHeap)
	{
		User::CompressAllHeaps();
		User::Heap().Compress();
		iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else if (aCommand == cmdRefresh)
	{
		iTaskView->refreshTaskList(TODO_REFRESH);
	}
	else if (aCommand == cmdNoHotkey)
	{
		configData->hotkeyEnabled = !configData->hotkeyEnabled;
		static_cast<CSMan2Document*>(Document())->saveConfig();
		iTaskView->ToggleHotKey();
	}
	else if (aCommand == cmdResolveName)
	{
		configData->resolveName = !configData->resolveName;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	/*************************************************************
	*
	* Commands for Bluejack view
	*
	**************************************************************/
	else if (aCommand == cmdBluejackClearLog)
	{
		iBJackView->oScanner->logBuffer.Copy(_L(""));
		iBJackView->oScanner->logHelper->updateLog(NULL, 0, CLogger::logAll);
	}
	else if (aCommand == cmdBluejackStop)
	{
		iBJackView->oScanner->stopScan();
	}
	else if (aCommand == cmdBluejackOne)
	{
		if (!iBJackView->oScanner->IsActive())
		{
			// bluejack one
			iBJackView->oScanner->scannerBehaviour = CScanner::behaviourBluejackOne;
			iBJackView->oScanner->startScan();
		}
	}
	else if (aCommand == cmdBluejackDelFileLog)
	{
		EikFileUtils::DeleteFile(iBJackView->oScanner->logHelper->logFilename);
		CEikonEnv::Static()->InfoMsg(R_TBUF_MISC_DONE);
	}
	else if (aCommand == cmdBluejackViewFileLog)
	{
		if (ConeUtils::FileExists(iBJackView->oScanner->logHelper->logFilename))
			QikFileUtils::StartAppL(iBJackView->oScanner->logHelper->logFilename);
		else
			CEikonEnv::Static()->InfoMsg(R_TBUF_BJACKLOGMISSING);
	}
	else if (aCommand == cmdBluejackSysMsg)
	{
		CBluejackNotifications* notificationDialog = new (ELeave) CBluejackNotifications(configData);
		if (notificationDialog->ExecuteLD(R_DIALOG_NOTIFICATION) == EEikBidOk)
			static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdBluejackOnce)
	{
		configData->proxSweepOnce = !configData->proxSweepOnce;
		static_cast<CSMan2Document*>(Document())->saveConfig();
	}
	else if (aCommand == cmdAbout)
	{	
		CEikDialog* aboutDialog = new (ELeave) CEikDialog;
		aboutDialog->ExecuteLD(R_DIALOG_ABOUT);
		
/*
#include <msvapi.h>
#include <mtclreg.h>
#include <msvstd.h>
LIB - msgs.lib

CMsvSession* iMsvSession = CMsvSession::OpenSyncL(*this);
// Create client registry object
CClientMtmRegistry* iClientMtmRegistry =
CClientMtmRegistry::NewL(*iMsvSession);

//handle also the invisible entries
TMsvSelectionOrdering sort;
sort.SetShowInvisibleEntries(ETrue);

// Search MMSs in the INBOX
CMsvEntry* parentEntry;
parentEntry = CMsvEntry::NewL* (iMsvSession,KMsvDraftEntryId, sort);
CleanupStack::PushL(parentEntry);

CMsvEntrySelection* entries = parentEntry->ChildrenWithMtmL(KUidMsgTypeMMS);
CleanupStack::PushL(entries);

entries->Count();
*/

/*		
RTelServer server;
RMobilePhone phone;
User::LeaveIfError( server.Connect() );
// load a phone profile
_LIT(KGsmModuleName, "erigsm.tsy");
User::LeaveIfError(server.LoadPhoneModule(KGsmModuleName));

// initialize the phone object
RTelServer::TPhoneInfo info;
User::LeaveIfError(server.GetPhoneInfo(0, info));
User::LeaveIfError(phone.Open(server, info.iName));

TRequestStatus tempStatus;
tempStatus = KRequestPending;
TInt32 sigStr;
TInt8 aBar;
phone.GetSignalStrength(tempStatus, sigStr, aBar);
User::WaitForRequest(tempStatus);
TBuf<50> temp;
temp.Format(_L("Signal str %d"), sigStr);
User::InfoPrint(temp);

tempStatus = KRequestPending;
RMobilePhone::TMobilePhoneLocationAreaV1 locArea;
RMobilePhone::TMobilePhoneNetworkInfoV1 pkgData;
TPckg<RMobilePhone::TMobilePhoneNetworkInfoV1Pckg> thePackage(pkgData);
phone.GetCurrentNetwork(tempStatus, thePackage, locArea);
User::WaitForRequest(tempStatus);

temp.Format(_L("Cellid %d"), locArea.iCellId);
User::InfoPrint(temp);

server.Close();
*/
//CEikonEnv::Static()->AppUiFactory()->MenuBar()->OverrideColorL(EColorMenuPaneDimmedText, KRgbBlack);
//CEikonEnv::Static()->AppUiFactory()->MenuBar()->SetTitleDimmed(0, ETrue);

	}
	else if (aCommand == cmdBluejackMsg)
	{
		if (!iBJackView->oScanner->IsActive())
		{
			CBluejackMessageDialog* bluejackMsgDialog = new (ELeave) CBluejackMessageDialog(iBJackView->oScanner);
			bluejackMsgDialog->ExecuteLD(R_BLUEJACK_VCARD);
		}
		else
			iEikonEnv->InfoMsg(R_TBUF_STOPSCANNER);
	}
	else if (aCommand == cmdBluejack)
	{
		if (!iBJackView->oScanner->IsActive())
		{
			iBJackView->oScanner->scannerBehaviour = CScanner::behaviourProximitySweep;
			iBJackView->oScanner->startScan();
		}
		else
			/* Note, this part of the else block will only get executed if
			1. You are in flip closed mode
			2. You chose to stop bluejack
			3. Bluejack was running
			This is because we disable this command in the fullscreen mode's toolbar if bluejack is already
			active i.e. you will not be able to press the button and issue this command in that scenario
			*/
			iBJackView->oScanner->stopScan();
	}
}

void CSMan2AppUi::launchApp(TUid appUid, TFileName docName, TBool putHistory)
{
	RApaLsSession mySession;
	TInt runResult;
	TApaAppInfo aInfo;		
	
	// Don't launch if another copy is already running.

	runResult = KErrGeneral;
	TApaTaskList taskList(CEikonEnv::Static()->WsSession());
	TApaTask theTask = taskList.FindApp(appUid);
	if (theTask.Exists())
	{
		// Why the *toot* did I even put this here????? This line causes problems!!
		//SetAppPosition(EFalse);
		
		theTask.BringToForeground();
		runResult = KErrNone;
	}
	else
	{
		if (FindApp(appUid, &aInfo) == 0)
		{
			if (docName.Compare(_L("")) == 0)
			{
				TRAPD(err, QikFileUtils::MakeAppDocumentFileNameL(docName, appUid));
				if (err != KErrNone)
					docName.Copy(_L(""));
			}
			CApaCommandLine* cmdln = CApaCommandLine::NewLC();
			cmdln->SetLibraryNameL(aInfo.iFullName);
			cmdln->SetCommandL(EApaCommandRun);
			cmdln->SetDocumentNameL(docName);
			mySession.Connect();
			runResult = mySession.StartApp(*cmdln);
			CleanupStack::PopAndDestroy(cmdln);
			mySession.Close();
		}
	}
	if (runResult != KErrNone)
		iEikonEnv->InfoMsg(R_TBUF_ERRLAUNCHAPP);
	else
	{
		if (putHistory)
		{
			TInt i, j;

			for (i = NUM_LASTRUN_HISTORY - 1; i > -1; i--)
				if (historyUidList->At(i) == appUid)
					break;
				
			if (i > -1)
				j = i;
			else
				j = NUM_LASTRUN_HISTORY - 1;

			for (; j > 0; j--)
				historyUidList->At(j) = historyUidList->At(j - 1);
			historyUidList->At(0) = appUid;
		}
	}
}

void CSMan2AppUi::SetAppPosition(TBool foreground)
{
	RWsSession& oWindowSession = iEikonEnv->WsSession();
	RWindowGroup& oWindowGroup = iEikonEnv->RootWin();
	
	int windowID = oWindowGroup.Identifier();
	TApaTask oTApaTask(oWindowSession);
	oTApaTask.SetWgId(windowID);
	if (foreground)
		oTApaTask.BringToForeground();
	else
		oTApaTask.SendToBackground();
}

void CSMan2AppUi::focusPhoneApp()
{
	TVwsViewId viewId;
	viewId.iAppUid = KUidPhoneApp;
	if (GetDefaultViewId(viewId) == KErrNone)
	{
		TRAPD(err, ActivateViewL(viewId));
		if (!err)
			iCoeEnv->RootWin().SetOrdinalPosition(-1);
	}
}

void CSMan2AppUi::HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination)
{
	if ((aEvent.Type() == EEventKey) && (aEvent.Key()->iCode == sCaptureKeyCodes[configData->iHotkeyDialogResult]))
	{
		if (CEikonEnv::Static()->ScreenDevice()->CurrentScreenMode() != 0)
			ActivateViewL(TVwsViewId(KUidSMan2App, KUidFlipclosedView));
		else 
		{
			TVwsViewId currentView;
			
			if (GetActiveViewId(currentView) != KErrNone)
				SetAppPosition(ETrue);
			else
			{
				if ((!CEikonEnv::Static()->AppUiFactory()->MenuBar()->DrawableWindow()->IsFaded()) && (appInForeground))
				{
					TInt selectedTitle;

					if (CEikonEnv::Static()->AppUiFactory()->MenuBar()->IsFocused())
					{
						selectedTitle = CEikonEnv::Static()->AppUiFactory()->MenuBar()->SelectedTitle();
						if (selectedTitle == (CEikonEnv::Static()->AppUiFactory()->MenuBar()->MenuTitleArray()->Count() - 2))
							ProcessCommandL(EEikCmdCanceled);
						else
						{
							if (selectedTitle == (CEikonEnv::Static()->AppUiFactory()->MenuBar()->MenuTitleArray()->Count() - 1))
								selectedTitle = -1;
							CEikonEnv::Static()->AppUiFactory()->MenuBar()->MoveHighlightToL(selectedTitle + 1, -1);
						}
					}
					else
					{
						CEikonEnv::Static()->AppUiFactory()->MenuBar()->MoveHighlightToL(CEikonEnv::Static()->AppUiFactory()->MenuBar()->MenuTitleArray()->Count() - 1,  -1);
					}
				}
				else
					SetAppPosition(ETrue);
			}
		}
	} 
	else if (aEvent.Type() == EEventScreenDeviceChanged)
	{
		// For another strange reason, i found that this event is fired off by the
		// phone when you launch your program in flipclosed mode. I didn't even open/close
		// the flip and yet it is fired....why?!?!? my workaround is to use "oldScreenMode"
		if (CEikonEnv::Static()->ScreenDevice()->CurrentScreenMode() == 0) // 0 = flip open
		{
			if (configData->iHotkeyDialogResult == 3)
				SetAppPosition(ETrue);
			oldScreenMode = 0;
		}
		else
		{
			if (oldScreenMode != 1)
			{
				if (configData->flipFlush)
					flushTasks();
				//focusPhoneApp();
				SetAppPosition(EFalse);
				oldScreenMode = 1;
			}
		}
	}	
	CQikAppUi::HandleWsEventL(aEvent, aDestination);
}

void CSMan2AppUi::flushTasks(void)
{
	if (iTaskView->snapshotTasks->Count() <= 0)
	{
		iEikonEnv->InfoMsg(R_TBUF_NOSNAPSHOT);
	}
	else
	{
		iTaskView->refreshTaskList(TODO_FLUSH);
		iEikonEnv->InfoMsg(R_TBUF_TASKSFLUSHED);
	}
}

void CSMan2AppUi::updateSnapShot(void)
{
	iTaskView->snapshotTasks->Reset();
	iTaskView->refreshTaskList(TODO_SNAPSHOT);
	if (iTaskView->saveSnapShot() == 0)
		iEikonEnv->InfoMsg(R_TBUF_SNAPSHOTUPDATED);
	else
		iEikonEnv->InfoMsg(R_TBUF_ERRSAVESNAPSHOT);
}

void CSMan2AppUi::FindAndLaunchAppNoUid(const TDesC *appPath)
{
	// Given the path to an .APP or .EXE file, find the UID of the app and launch it
	RApaLsSession mySession;
	TApaAppCapabilityBuf aCapabilityBuf;
	TApaAppInfo aInfo;
	TBool appRegistered = EFalse;

	TFullName temp;
	temp.Copy(*appPath);
	temp.UpperCase();
	TParsePtr toFind(temp);
	
	// Bounce the app server
	FindApp(KUidControlPanel, &aInfo);
	
	mySession.Connect();
	mySession.GetAllApps();
	while (mySession.GetNextApp(aInfo) == KErrNone)
	{
		if (mySession.GetAppCapability(aCapabilityBuf, aInfo.iUid) == KErrNone)
		{
			// Only scan applications that are not hidden
			if (aCapabilityBuf().iAppIsHidden == EFalse)
			{
				aInfo.iFullName.UpperCase();
				if (aInfo.iFullName == toFind.FullName())
				{
					launchApp(aInfo.iUid, _L(""), EFalse);
					appRegistered = ETrue;
					break;
				}
			}
		}
	}
	mySession.Close();
	if (!appRegistered)
		iEikonEnv->InfoMsg(R_TBUF_FILEMAN_APPNOTREGISTERED);
}

TInt CSMan2AppUi::FindApp(TUid appUid, TApaAppInfo *aInfo)
{
	// Given an app UID, find if it is installed and get its app info

	RApaLsSession mySession;
	TApaAppInfo theAppInfo;
	TApaAppCapabilityBuf aCapabilityBuf;
	TInt errRes, iTimeout = 50;
	TTimeIntervalMicroSeconds32 iDelay = 100;
	
	// For some strange reason when I have just issued a command to write to the
	// disk (via saving snapshot or saving shortcuts), the app server always returns
	// an invalid application list (corrupted). I have to wait a while before the query
	// runs correct again....?????

	CEikonEnv::Static()->BusyMsgL(R_TBUF_VALIDATING);
	//_L("Validating..."), TGulAlignment(EHRightVTop));
	mySession.Connect();
	mySession.GetAllApps();
	errRes = 2;
	while ((errRes != 0) & (iTimeout > 0))
	{
		while (mySession.GetNextApp(theAppInfo) == KErrNone)
		{
			errRes = 1;
			if (mySession.GetAppCapability(aCapabilityBuf, theAppInfo.iUid) == KErrNone)
			{
				// Only scan applications that are not hidden
				if (aCapabilityBuf().iAppIsHidden == EFalse)
				{
					if (theAppInfo.iUid == appUid)
					{
						aInfo->iUid = theAppInfo.iUid;
						aInfo->iFullName = theAppInfo.iFullName;
						aInfo->iCaption = theAppInfo.iCaption;
						errRes = 0;
						break;
					}
				}
			}
		}
		User::After(iDelay);
		iTimeout--;
	} 
	mySession.Close();
	CEikonEnv::Static()->BusyMsgCancel();
	return errRes;
}

void CSMan2AppUi::sizeToText(TInt64 size, TDes* desc)
{
	TInt64 origSize, decimals;
	TBuf<3> sizeName;
	
	origSize = size;
	if (size < 1024)
	{
		sizeName = _L("b)");
		decimals = 0;
	}
	else if (size < (1024 * 1024))
	{
		size = size / 1024;
		decimals = (origSize - (size * 1024)) / 1024;
		sizeName = _L("KB)");
	}
	else
	{
		size = size / (1024 * 1024);
		decimals = (origSize - (size * 1024 * 1024)) / 1024;
		sizeName = _L("MB)");
	}
	
	desc->Num(size);
	desc->Append(_L("."));
	desc->AppendFormat(_L("%2d"), decimals);
	if (size > 999)
		desc->Insert(desc->Length() - 6, _L(","));
	desc->Append(sizeName);
	desc->Insert(0, _L("]("));
}

void CSMan2AppUi::DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane)
{
	if (aMenuId == R_GENERIC_FN)
	{
		if ((configData->fnActivatedMenus & 1) == 0)
			aMenuPane->DeleteMenuItem(cmdSwitchFile);
		if ((configData->fnActivatedMenus & 2) == 0)
			aMenuPane->DeleteMenuItem(cmdSwitchSysInfo);
		if ((configData->fnActivatedMenus & 4) == 0)
			aMenuPane->DeleteMenuItem(cmdSwitchBJack);
		if ((configData->fnActivatedMenus & 8) == 0)
			aMenuPane->DeleteMenuItem(cmdSwitchDB);
		if ((configData->fnActivatedMenus & 16) == 0)
			aMenuPane->DeleteMenuItem(cmdSwitchAgenda);
	}
	else if (aMenuId == R_OPTIONS_AUTOSTART_MENU)
	{
		CAutostartDialog* autoStartDialog = new (ELeave) CAutostartDialog();
		autoStartDialog->loadBootFile();
		if (autoStartDialog->autoStartConfig.autoStartUid[MAX_AUTOSTART - 1] == KUidSMan2App)
			aMenuPane->SetItemButtonState(cmdAutostartSMan, EEikMenuItemSymbolOn);
		delete autoStartDialog;
	}
	else if (aMenuId == R_FIXAPPORDER_MENU)
	{
		if (!ConeUtils::FileExists(KAppConfigFileBackup))
			aMenuPane->SetItemDimmed(cmdUtilsFixAppOrderRestore, ETrue);
	}
	else if (viewId == EViewAgenda)
	{
		if (aMenuId == R_AGENDA_COMPLETED_MENU)
		{
			if (configData->agendaShowCompleteCalendar)
				aMenuPane->SetItemButtonState(cmdAgendaCompleteCalendar, EEikMenuItemSymbolOn);
			if (configData->agendaShowCompleteTodo)
				aMenuPane->SetItemButtonState(cmdAgendaCompleteTodo, EEikMenuItemSymbolOn);
		}
		else if (aMenuId == R_AGENDA_AUTOEXPAND_MENU)
		{
			if (configData->agendaAutoExpandCalendar)
				aMenuPane->SetItemButtonState(cmdAgendaAutoExpandCalendar, EEikMenuItemSymbolOn);
			if (configData->agendaAutoExpandTodo)
				aMenuPane->SetItemButtonState(cmdAgendaAutoExpandTodo, EEikMenuItemSymbolOn);
		}
	}
	else if (viewId == EViewDB)
	{
		if (aMenuId == R_DB_OPTIONS)
		{
			if (!iDBView->dbIsOpened)
			{
				aMenuPane->SetItemDimmed(cmdDBUpdateStats, ETrue);
				aMenuPane->SetItemDimmed(cmdDBCompact, ETrue);
				aMenuPane->SetItemDimmed(cmdDBChangePassword, ETrue);
				aMenuPane->SetItemDimmed(cmdDBSortAsc, ETrue);
				aMenuPane->SetItemDimmed(cmdDBSortDes, ETrue);
				aMenuPane->SetItemDimmed(cmdDBCloseDB, ETrue);
			}
			else if (iDBView->dbIsOpened)
			{
				aMenuPane->SetItemDimmed(cmdDBReadDB, ETrue);
				if (iDBView->currentDBView != CSMan2DBView::DBView_List)
				{
					aMenuPane->SetItemDimmed(cmdDBChangePassword, ETrue);
					aMenuPane->SetItemDimmed(cmdDBReadDB, ETrue);
					aMenuPane->SetItemDimmed(cmdDBReInit, ETrue);
					aMenuPane->SetItemDimmed(cmdDBSortAsc, ETrue);
					aMenuPane->SetItemDimmed(cmdDBSortDes, ETrue);
					aMenuPane->SetItemDimmed(cmdDBUpdateStats, ETrue);
					aMenuPane->SetItemDimmed(cmdDBCompact, ETrue);
				}
			}
			if (configData->dbSortAscending)
				aMenuPane->SetItemButtonState(cmdDBSortAsc, EEikMenuItemSymbolOn);
			else
				aMenuPane->SetItemButtonState(cmdDBSortDes, EEikMenuItemSymbolOn);
		}
		else if (aMenuId == R_DB_DOBUTTON)
		{
			if (!iDBView->dbIsOpened)
			{
				aMenuPane->SetItemDimmed(cmdDBAdd, ETrue);
				aMenuPane->SetItemDimmed(cmdDBEdit, ETrue);	
				aMenuPane->SetItemDimmed(cmdDBView, ETrue);
				aMenuPane->SetItemDimmed(cmdDBDel, ETrue);
				aMenuPane->SetItemDimmed(cmdDBSave, ETrue);
			}
			else
			{
				switch (iDBView->currentDBOp)
				{
					case CSMan2DBView::DBOp_Browsing:
					{
						aMenuPane->SetItemDimmed(cmdDBSave, ETrue);
						break;
					}
					case ((CSMan2DBView::DBOp_Editing) || (CSMan2DBView::DBOp_Adding)):
					{
						aMenuPane->SetItemDimmed(cmdDBAdd, ETrue);
						aMenuPane->SetItemDimmed(cmdDBEdit, ETrue);	
						aMenuPane->SetItemDimmed(cmdDBDel, ETrue);
						aMenuPane->SetItemDimmed(cmdDBView, ETrue);
						break;
					}
					case CSMan2DBView::DBOp_Adding:
					{
						aMenuPane->SetItemDimmed(cmdDBAdd, ETrue);	
						aMenuPane->SetItemDimmed(cmdDBEdit, ETrue);	
						aMenuPane->SetItemDimmed(cmdDBDel, ETrue);
						aMenuPane->SetItemDimmed(cmdDBView, ETrue);
						break;
					}
					case CSMan2DBView::DBOp_Viewing:
					{
						aMenuPane->SetItemDimmed(cmdDBAdd, ETrue);	
						aMenuPane->SetItemDimmed(cmdDBDel, ETrue);
						aMenuPane->SetItemDimmed(cmdDBView, ETrue);
						aMenuPane->SetItemDimmed(cmdDBSave, ETrue);
						break;
					}
				}
			}
		}
	}
	else if (viewId == EViewFile)
	{
		if (aMenuId == R_FILE_DRIVES_MENU)
		{
			// Get volume + free space information for all the drives.
			
			TVolumeInfo volInfo;
			TInt err, i;
			TBuf<KMaxFileName + 30> desc;
			TBuf<3> drivePrefix;
			CEikMenuPaneItem::SData itemData;
			
			for (i = 0; i < KMaxDrives; i++)
			{
				err = iEikonEnv->FsSession().Volume(volInfo, EDriveA + i);
				if (err == KErrNone)
				{
					if (volInfo.iDrive.iType != EMediaRom)
						sizeToText(volInfo.iFree, &desc);
					else
						desc.Copy(_L("](ROM)"));
						
					desc.Insert(0, volInfo.iName);
					drivePrefix.Format(_L("%c:["), 0x41 + i);
					desc.Insert(0, drivePrefix);
					 
					itemData.iText.Copy(desc);
					itemData.iCommandId = cmdFileMan_Drive_A + i;
					itemData.iFlags = 0;
					itemData.iCascadeId = 0;
					aMenuPane->AddMenuItemL(itemData);					
				}				
			}
		}
		else if (aMenuId == R_FILE_OPTIONS_MENU)
		{
			if (configData->beamProtocol == CBlueBeam::protocolIrDA)
				aMenuPane->SetItemButtonState(cmdFileMan_IrDA, EEikMenuItemSymbolOn);
			else if (configData->beamProtocol == CBlueBeam::protocolBluetooth)
				aMenuPane->SetItemButtonState(cmdFileMan_Bluetooth, EEikMenuItemSymbolOn);
				
			if (configData->showHiddenFiles)
				aMenuPane->SetItemButtonState(cmdFileManToggleHiddenView, EEikMenuItemSymbolOn);
			if (configData->autoRunApp)
				aMenuPane->SetItemButtonState(cmdFileManToggleRunApp, EEikMenuItemSymbolOn);
			if (configData->autoOpenFile)
				aMenuPane->SetItemButtonState(cmdFileManToggleOpenFile, EEikMenuItemSymbolOn);
		}
	}
	else if (viewId == EViewBJack)
	{
		if (aMenuId == R_OPTIONS_BLUEJACK_MENU)
		{
			if (configData->showBluejackSysMsg)
				aMenuPane->SetItemButtonState(cmdBluejackSysMsg, EEikMenuItemSymbolOn);
			if (configData->resolveName)
				aMenuPane->SetItemButtonState(cmdResolveName, EEikMenuItemSymbolOn);
			if (configData->autoSave)
				aMenuPane->SetItemButtonState(cmdBluejackAutoSave, EEikMenuItemSymbolOn);
			if (configData->proxSweepOnce)
				aMenuPane->SetItemButtonState(cmdBluejackOnce, EEikMenuItemSymbolOn);
		}
		else if (aMenuId == R_BLUEJACK_FILELOG_MENU)
		{
			if (iBJackView->oScanner->IsActive())
			{
				aMenuPane->SetItemDimmed(cmdBluejackDelFileLog, ETrue);
				aMenuPane->SetItemDimmed(cmdBluejackViewFileLog, ETrue);
			}
		}
	}
	else if (viewId == EViewTask)
	{
		if (aMenuId == R_EXTRAS_MENU)
		{
			//if ((iBJackView->oScanner->IsActive()) || (iFileView->blueBeam->IsActive()))
			//	aMenuPane->SetItemDimmed(cmdFileMan, ETrue);
			if (!configData->hotkeyEnabled)
				aMenuPane->SetItemButtonState(cmdNoHotkey, EEikMenuItemSymbolOn);
			if (configData->flipFlush)
				aMenuPane->SetItemButtonState(cmdFlipFlush, EEikMenuItemSymbolOn);
			if (configData->forceEndTask)
				aMenuPane->SetItemButtonState(cmdForceEndTask, EEikMenuItemSymbolOn);
			if (configData->showHiddenTasks)
				aMenuPane->SetItemButtonState(cmdShowHiddenTask, EEikMenuItemSymbolOn);
			if (configData->persist)
				aMenuPane->SetItemButtonState(cmdPersist, EEikMenuItemSymbolOn);
		}
		else if (aMenuId == R_RUNAPPLIST_MENU)
		{
			CEikMenuPaneItem::SData itemData;

			CShortcutDialog* shortcutDialog = new (ELeave) CShortcutDialog(NULL);
			shortcutDialog->populateAppArray();
			startUidList->Reset();
			startUidList->Compress();
			for (int i = 0; i < shortcutDialog->appListArray->MdcaCount(); i++)
			{
				itemData.iText.Copy(shortcutDialog->appListArray->MdcaPoint(i));
				itemData.iCommandId = cmdRunAppBase + i;
				startUidList->AppendL(shortcutDialog->appListUid->At(i));
				itemData.iFlags = 0;
				itemData.iCascadeId = 0;
				aMenuPane->AddMenuItemL(itemData);
			}
			delete shortcutDialog;

		}
		else if (aMenuId == R_SHORTCUTS_MENU)
		{				
			// This is faster than NUM_SHORTCUTS calls to FindApp() as long as NUM_SHORTCUTS is less
			// than the number of apps u have installed. However, this is bulkier in code.
			RApaLsSession mySession;
			TApaAppCapabilityBuf aCapabilityBuf;
			TApaAppInfo aInfo;
			TInt i;
			CEikMenuPaneItem::SData itemData;
			
			// Bounce the app server
			FindApp(KUidControlPanel, &aInfo);
			
			mySession.Connect();
			mySession.GetAllApps();
			while (mySession.GetNextApp(aInfo) == KErrNone)
			{
				if (mySession.GetAppCapability(aCapabilityBuf, aInfo.iUid) == KErrNone)
				{
					// Only scan applications that are not hidden
					if (aCapabilityBuf().iAppIsHidden == EFalse)
					{
						for (i = 0; i < NUM_SHORTCUTS; i++)
						{
							if (aInfo.iUid == configData->shortCutUid[i])
							{
								aMenuPane->SetItemDimmed(cmdShort1 + i,EFalse);
								aMenuPane->SetItemTextL(cmdShort1 + i, aInfo.iCaption);
							}
						}
					}
				}
			}
			mySession.Close();

			// Dynamically initialize the history menu
			TBool hasHistory = EFalse;
			TInt iMenuPos;
			
			for (i = 0; i < NUM_LASTRUN_HISTORY; i++)
			{
				if (historyUidList->At(i) != KNullUid)
				{
					if (FindApp(historyUidList->At(i), &aInfo) == 0)
						itemData.iText.Copy(aInfo.iCaption);
					else
						itemData.iText=_L("<none>");

					itemData.iCommandId = cmdLastRunBase + i;
					itemData.iFlags = 0;
					itemData.iCascadeId = 0;
					// add the item
					aMenuPane->AddMenuItemL(itemData);
					hasHistory = ETrue;
				}
			}
			
			// As per Phil Spencer's rant, I'm now in compliance with UIQ's style guide. LOL. :)
			if (hasHistory)
			{
				CEikMenuPaneItem *theMenuItem = aMenuPane->ItemAndPos(cmdShort7, iMenuPos);
				theMenuItem->iData.iFlags = theMenuItem->iData.iFlags | EEikMenuItemSeparatorAfter;
			}
		}
	}
	else if (viewId == EViewFlipClosed)
	{
		if (aMenuId == R_FC_BLUEJACK_MENU)
		{
			if (iBJackView->oScanner->IsActive())
			{
				HBufC* dataBuffer = CEikonEnv::Static()->AllocReadResourceL(R_TBUF_MISC_FC_BJACK_SWEEP);
				aMenuPane->SetItemTextL(cmdBluejack, *dataBuffer);
				delete dataBuffer;
			}
		}
	}
}

/*************************************************************
*
* External interface
*
**************************************************************/

TUid CSMan2Application::AppDllUid() const
{
	return KUidSMan2App;
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CSMan2Application;
}
