#include <qikapplication.h>		// CQikApp
#include <qikappui.h>			// CQikAppUi
#include <quartzkeys.h>			// EQuartzKey....Confirm, TwoWayDown, TwoWayUp
#include <qikzoomdlg.h>			// CQikZoomDialog
#include <QPhoneAppExternalInterface.h>	// KUidPhoneApp
#include <qikdocument.h>		// CQikDocument
#include <eikdll.h>			// StartExeL

#include "dbview.h"
#include "bjackview.h"
#include "dialogs.h"
#include "config.h"
#include "fileman.h"
#include "viewbase.h"
#include "flipcloseview.h"
#include "tasklistview.h"
#include "filelistview.h"
#include "sysinfoview.h"
#include "agendaview.h"

#if !defined(__SMAN_H__)
#define __SMAN_H__

#define ZOOM_INCREMENT 500

// These 3 literals are used by the utilities to fix app ordering
_LIT(KAppConfigFileBackup, "C:\\SYSTEM\\APPS\\ALAUNCH\\Alaunch.bak");
_LIT(KAppConfigFile, "C:\\SYSTEM\\APPS\\ALAUNCH\\Alaunch.ini");
_LIT(KAppTaskName, "Applications");

static const TUint sCaptureKeyCodes[] = 
{
	0,						// Disable - unused. Place holder so indexing is correct.
	EKeyApplication1,			// Camera
	EKeyApplication0,			// Internet
	0						// Flip open - unused. Place holder so indexing is correct.
};

/*************************************************************
*
* Document class
*
**************************************************************/

class CSMan2Document : public CQikDocument
{
public:
	static CSMan2Document* NewL(CEikApplication& aApp);
	~CSMan2Document();
	CSMan2Document(CEikApplication& aApp);
	void ConstructL();
	CConfig configData;
	void saveConfig(void);
	TInt loadConfig(void);	
	TFileName configFileName;

private: // from CEikDocument
	CQikAppUi* CreateAppUiL();
	CVerInfo verData;
};

/*************************************************************
*
* App UI class
*
**************************************************************/

class CSMan2AppUi : public CQikAppUi
{	
public:
	enum viewType
	{
		EViewTask,
		EViewFile,
		EViewBJack,
		EViewDB,
		EViewSysInfo,
		EViewAgenda,
		EViewFlipClosed
	};
	CSMan2AppUi(CConfig *cData) { configData = cData; }
	void ConstructL();
	~CSMan2AppUi();
	void HandleForegroundEventL(TBool aForeground);
	void flushTasks(void);
	void HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination);
	void switchView(TInt viewId);
	CSMan2TaskListView* iTaskView;
	CSMan2FileListView* iFileView;
	CSMan2FlipCloseView* iFlipCloseView;
	CSMan2BJackView* iBJackView;
	CSMan2SysInfoView* iSysInfoView;
	CSMan2DBView* iDBView;
	CSMan2AgendaView* iAgendaView;
	void FindAndLaunchAppNoUid(const TDesC *appPath);
	TBool FindApp(TUid appUid, TApaAppInfo *aInfo);
	void SetAppPosition(TBool foreground);
	void focusPhoneApp();
	CSManFileMan *smanFileMan;
	TInt bjackIconLabelCurrentImage;
	void updateBJackIcon(TBool draw);
	void doCancelMenu(void);
	void doProcessCommand(TInt aCommand);
	TInt viewId;
	
private:
	void doFileMan(TInt op);
	TUint oldScreenMode;
	CArrayFixFlat<TUid> *startUidList, *historyUidList;
	void updateSnapShot(void);
	void sizeToText(TInt64 size, TDes* buffer);
	void launchControlPanel(TPtrC appletName);
	void launchApp(TUid appUid, TFileName docName, TBool putHistory);
	void SetZoom(TInt zoomLevel);
	void FileManGetFile(TPtrC *entryType, TPtrC *entryName);
	TBool PromptAppFixOrder(TInt aPromptResourceId);
	CConfig *configData;
	TBool appInForeground;
	TInt iOldMenuBarYPos;
	
protected:
	void DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane);
	void HandleCommandL(TInt aCommand);
};

/*************************************************************
*
* Application class
*
**************************************************************/

class CSMan2Application : public CQikApplication
{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
};

// to launch control panel CTL files. Thanks to Phil Spencer of symbian for the tip.
typedef void (*TLoadDll)(const TDesC& aDllFullPathName);

#endif